@@ -17,7 +17,8 @@ from celery.exceptions import NotRegistered
 from firexapp.engine.logging import add_hostname_to_log_records
 
 from firexkit.result import wait_on_async_results, disable_async_result, ChainRevokedException, \
-    mark_queues_ready, get_results, get_task_name_from_result, ChainRevokedPreRunException
+    mark_queues_ready, get_results, get_task_name_from_result, ChainRevokedPreRunException, \
+    monkey_patch_async_result_to_track_instances, is_async_result_monkey_patched_to_track, disable_all_async_results
 from firexkit.chain import InjectArgs, verify_chain_arguments, InvalidChainArgsException
 from firexapp.fileregistry import FileRegistry
 from firexapp.submit.uid import Uid
@@ -29,7 +30,7 @@ from firexapp.application import import_microservices, get_app_tasks, get_app_ta
 from firexapp.engine.celery import app
 from firexapp.broker_manager.broker_factory import BrokerFactory
 from firexapp.submit.shutdown import launch_background_shutdown
-from firexapp.submit.install_configs import load_new_install_configs, FireXInstallConfigs
+from firexapp.submit.install_configs import load_new_install_configs, FireXInstallConfigs, INSTALL_CONFIGS_ENV_NAME
 from firexapp.submit.arguments import whitelist_arguments
 from firexapp.common import dict2str, silent_mkdir, create_link
 
@@ -144,13 +145,18 @@ class SubmitBaseApp:
                                    help="Create a symlink back the root of the run's logs directory")
         submit_parser.add_argument('--soft_time_limit', help="Task default soft_time_limit", type=int)
         submit_parser.add_argument('--install_configs', help="Path to JSON file specifying installation-wide configs",
-                                   type=str, default=None)
+                                   type=str, default=os.environ.get(INSTALL_CONFIGS_ENV_NAME, None))
         submit_parser.add_argument('--celery_concurrency', '--celery_work_slots',
                                    type=int, action=AdjustCeleryConcurrency,
                                    help='Number of worker slots in celery pool',
                                    # Some machines have loads of CPUs, so cap at 100.
                                    default=min([multiprocessing.cpu_count()*4, 100]))
         submit_parser.add_argument('--json_file', help='Link name for the report json file', action=JsonFileAction)
+        submit_parser.add_argument('--wait_tracking_services_release_console',
+                                   help='Wait for tracking services (e.g. Flame) to indicate they are ready to release '
+                                        'the console before doing so.', nargs='?', const=True,
+                                   default=True, action=OptionalBoolean,)
+
         submit_parser.set_defaults(func=self.run_submit)
 
         for service in get_tracking_services():
@@ -168,6 +174,8 @@ class SubmitBaseApp:
             self.init_file_logging()
             return self.submit(args, others)
         finally:
+            if args.wait_tracking_services_release_console:
+                self.wait_tracking_services_release_console_ready()
             self.copy_submission_log()
 
     @staticmethod
@@ -269,6 +277,12 @@ class SubmitBaseApp:
             self.main_error_exit_handler(reason=str(e))
             sys.exit(-1)
         self.wait_tracking_services_task_ready()
+
+        # AsyncResult objects cannot be in memory after the broker (i.e. backend) shutdowns, otherwise errors are
+        # produced when they are garbage collected. We therefore monkey patch AsyncResults to track all instances
+        # (e.g. from unpickle, instantiated directly, etc) so that disable_all_async_results can disable their
+        # references to the backend.
+        monkey_patch_async_result_to_track_instances()
         root_task_result_promise = root_task.s(submit_app=self, **chain_args).delay()
 
         self.copy_submission_log()
@@ -281,8 +295,6 @@ class SubmitBaseApp:
             self.self_destruct(chain_details=(root_task_result_promise, chain_args),
                                reason="Sync run: completed successfully")
 
-        self.wait_tracking_services_release_console_ready()
-
     def check_for_failures(self, root_task_result_promise, unsuccessful_services):
         if unsuccessful_services:
             msg, rc = format_unsuccessful_services(unsuccessful_services)
@@ -434,7 +446,10 @@ class SubmitBaseApp:
                 additional_chain_args.update(extra)
         return additional_chain_args
 
-    def wait_tracking_services_pred(self, service_predicate, description, timeout)->None:
+    def wait_tracking_services_pred(self, service_predicate, description, timeout) -> None:
+        if not self.enabled_tracking_services:
+            return
+
         services_by_name = {get_service_name(s): s for s in self.enabled_tracking_services}
         not_passed_pred_services = list(services_by_name.keys())
         start_wait_time = time.time()
@@ -446,12 +461,12 @@ class SubmitBaseApp:
                     # Service has passed the predicate, remove it from the list of not passed services.
                     not_passed_pred_services = [n for n in not_passed_pred_services if service_name != n]
                     if not not_passed_pred_services:
-                        logger.debug(f"Last tracking service up (long pole) is: {service_name}")
+                        logger.debug(f"Last tracking service {description} (long pole) is: {service_name}")
             if not_passed_pred_services:
                 time.sleep(0.1)
 
         if not_passed_pred_services:
-            logger.warning("The following services are still not %s after %s secs:" % (description, timeout))
+            logger.warning(f"The following services are still not {description} after {timeout} secs:")
             for s in not_passed_pred_services:
                 launch_file = getattr(services_by_name[s], 'stdout_file', None)
                 msg = f'{s}: see {launch_file}' if launch_file else s
@@ -488,7 +503,11 @@ class SubmitBaseApp:
                 # Under no circumstances should report generation prevent celery and broker cleanup
                 logger.error('Error in generating reports', exc_info=True)
             finally:
-                if chain_result:
+                # AsyncResult objects access self.backend when garbage collected. Since we're about to initiate a
+                # process to stop the backend, prevent all AsyncResult objects from accessing self.backend.
+                if is_async_result_monkey_patched_to_track():
+                    disable_all_async_results()
+                elif chain_result:
                     disable_async_result(chain_result)
 
         logger.debug("Running FireX self destruct")
